#ifndef _LEG_HPP
#define _LEG_HPP

/**
 * \file leg.hpp
 *
 * második előadás sablonjai
 * Sablonokat tartalamazó header fájloknak .hpp kiterjesztést szokás adni.
*/


/**
 * legElem függvénysablon
 * A paraméterként kapott tömb adott tulajdonságú elemét választja ki
 * A tulajdonságot a sel predikátumfüggvény határozza meg.
 * A sel(T, T) predikátumtól elvárjuk, hogy logikai környezetben
 * kiértékelhető legyen, és működjön a T típusra.
 * T-nek kell, hogy legyen másoló konstruktora és működjön az értékadás
 */
template <typename T, typename S>
T legElem(T a[], int n, S sel) {
    T tmp = a[0];
    for (int i = 1; i < n; ++i)
        if (sel(a[i], tmp)) tmp = a[i];
    return tmp;
}

/**
 * nagyobb_e függvénysablon
 * A függvény igaz értéket ad, ha az első paraméter nagyobb mint a második
 * A paraméterek között értelmezettnek kell lennie a > operátornak.
 * T-nek kell, hogy legyen másoló konstruktora
 */
template <typename T>
bool nagyobb_e(T a, T b) {
    return a > b;
}

/**
 * kisebb_e függvénysablon
 * A függvény igaz értéket ad, ha az első paraméter kisebb mint a második
 * A paraméterek között értelmezettnek kell lennie a < operátornak.
 * T-nek kell, hogy legyen másoló konstruktora
 */
template <typename T>
bool kisebb_e(T a, T b) {
    return a < b;
}

#endif // _LEG_HPP
